import argparse
import json
import random
from pathlib import Path

import yaml

from evrouting.osm.imports import read_osm

from evaluation.T import *
from evaluation.export import write_head
from evaluation.queries import *


def query_benchmark(graphs,
                    conf,
                    result_dir):
    query_conf = [
        Query(query_function=gasstation_query,
              filename='gasstation.csv',
              row_dataclass=GasstationQueryRow),
        Query(charge_query, 'charge.csv', ChargeQueryRow),
        Query(classic_query, 'classic.csv', ClassicQueryRow),
        Query(astar_query, 'astar.csv', AStarQueryRow)
    ]

    for G in graphs:
        nodes = random.choices(list(G.nodes), k=2 * conf['queries_per_setup'])
        for setup in conf['setups']:
            start_nodes = nodes[:int(len(nodes) / 2)]
            target_nodes = nodes[int(len(nodes) / 2):]

            for func, filename, row_class in query_conf:
                with result_dir.joinpath(filename).open('w') as f:
                    write_head(f, row_class)
                    for s, t in zip(start_nodes, target_nodes):
                        func(G, setup, s, t, f)


def get_map(osm_path: Path, cs_path: Path):
    graph = read_osm(str(osm_path))

    with cs_path.open() as f:
        cs = json.load(f)
    graph.insert_charging_stations(cs)

    return graph


def apply_conversions(conf):
    """kWh to Wh"""
    for setup in conf['setups']:
        setup['capacity'] = 1000 * setup['capacity']
        setup['consumption']['consumption_coefficient'] = 1000 * setup['consumption']['consumption_coefficient']
        setup['mu_s'] = 1000 * setup['mu_s']
        setup['mu_t'] = 1000 * setup['mu_t']
    return conf


if __name__ == '__main__':
    base = Path(__file__).parent
    results_dir = base.joinpath('results')
    static_dir = base.joinpath('static')
    maps_dir = static_dir.joinpath('maps')

    # Charging Stations
    cs_path = static_dir.joinpath('charging_stations.json')

    parser = argparse.ArgumentParser(description='Run Benchmark Scripts.')
    parser.add_argument(
        '--configs',
        help='List of filenames to benchmark YAML configs in ./configs.',
        type=Path,
        nargs='+'
    )

    args = parser.parse_args()
    path: Path
    for path in args.configs:
        benchmark_dir = results_dir.joinpath(path.with_suffix(''))
        benchmark_dir.mkdir(exist_ok=True)

        path = path.with_suffix('.yaml')

        with base.joinpath('configs/', path).open() as f:
            conf = apply_conversions(yaml.load(f, Loader=yaml.Loader))

        graphs = [get_map(maps_dir.joinpath(m), cs_path) for m in conf['maps']]

        if conf['type'] == 'query':
            query_dir = benchmark_dir.joinpath('queries')
            query_dir.mkdir(exist_ok=True)
            query_benchmark(graphs=graphs,
                            conf=conf,
                            result_dir=query_dir)
